@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  Copyright (c) 2003 Christian Szegedy <csdontspam871@metamatix.org>    *)
 (*                                                                        *)
-(*  Copyright (c) 2007 Jane Street Capital, LLC                           *)
+(*  Copyright (c) 2007 Jane Street Holding, LLC                           *)
 (*                     Author: Markus Mottl <markus.mottl@gmail.com>      *)
 (*                                                                        *)
 (*  Permission is hereby granted, free of charge, to any person           *)
@@ -114,6 +114,12 @@ module Data = struct
     | BLOB of string
 
   let to_string = function
+    | NONE | NULL -> ""
+    | INT i -> Int64.to_string i
+    | FLOAT f -> string_of_float f
+    | TEXT t | BLOB t -> t
+
+  let to_string_debug = function
     | NONE -> "NONE"
     | NULL -> "NULL"
     | INT i -> sprintf "INT <%Ld>" i
@@ -160,6 +166,7 @@ external finalize : stmt -> Rc.t = "caml_sqlite3_stmt_finalize"
 external expired : stmt -> bool = "caml_sqlite3_expired"
 
 external data_count : stmt -> int = "caml_sqlite3_data_count"
+external column_count : stmt -> int = "caml_sqlite3_column_count"
 external column : stmt -> int -> Data.t = "caml_sqlite3_column"
 external column_name : stmt -> int -> string = "caml_sqlite3_column_name"
 
