@@ -38,6 +38,22 @@
 
 #include <sqlite3.h>
 
+#if __GNUC__ >= 3
+# define inline inline __attribute__ ((always_inline))
+# if !defined(__FreeBSD__) && !__APPLE__
+# define __unused __attribute__ ((unused))
+# endif
+#else
+# define __unused
+# define inline
+#endif
+
+#if SQLITE_VERSION_NUMBER >= 3003009
+# define my_sqlite3_prepare sqlite3_prepare_v2
+#else
+# define my_sqlite3_prepare sqlite3_prepare
+#endif
+
 /* Utility definitions */
 
 static const value v_None = Val_int(0);
@@ -78,7 +94,7 @@ static value *caml_sqlite3_InternalError = NULL;
 static value *caml_sqlite3_Error = NULL;
 static value *caml_sqlite3_RangeError = NULL;
 
-static inline value raise_with_two_args(value v_tag, value v_arg1, value v_arg2)
+static inline void raise_with_two_args(value v_tag, value v_arg1, value v_arg2)
 {
   CAMLparam3(v_tag, v_arg1, v_arg2);
   value v_exc = caml_alloc_small(3, 0);
@@ -93,7 +109,7 @@ static inline void raise_sqlite3_InternalError(char *msg) Noreturn;
 
 static inline void raise_sqlite3_InternalError(char *msg)
 {
-  raise_with_string(*caml_sqlite3_InternalError, msg);
+  caml_raise_with_string(*caml_sqlite3_InternalError, msg);
 }
 
 static inline void range_check(int v, int max)
@@ -102,9 +118,9 @@ static inline void range_check(int v, int max)
     raise_with_two_args(*caml_sqlite3_RangeError, Val_int(v), Val_int(max));
 }
 
-static inline void raise_sqlite3_Error(const char *fmt, ...) Noreturn;
+static void raise_sqlite3_Error(const char *fmt, ...) Noreturn;
 
-static inline void raise_sqlite3_Error(const char *fmt, ...)
+static void raise_sqlite3_Error(const char *fmt, ...)
 {
   char buf[1024];
   va_list args;
@@ -116,7 +132,7 @@ static inline void raise_sqlite3_Error(const char *fmt, ...)
   caml_raise_with_string(*caml_sqlite3_Error, buf);
 }
 
-static inline void raise_sqlite3_misuse_db(db_wrap *dbw, const char *fmt, ...)
+static void raise_sqlite3_misuse_db(db_wrap *dbw, const char *fmt, ...)
 {
   char buf[1024];
   va_list args;
@@ -143,7 +159,7 @@ static inline void check_db(db_wrap *dbw, char *loc)
     raise_sqlite3_misuse_db(dbw, "Sqlite3.%s called with closed database", loc);
 }
 
-static inline void raise_sqlite3_misuse_stmt(const char *fmt, ...)
+static void raise_sqlite3_misuse_stmt(const char *fmt, ...)
 {
   char buf[1024];
   va_list args;
@@ -171,7 +187,7 @@ static inline stmt_wrap * safe_get_stmtw(char *loc, value v_stmt)
 
 /* Initialisation */
 
-CAMLprim value caml_sqlite3_init(value v_unit)
+CAMLprim value caml_sqlite3_init(value __unused v_unit)
 {
   caml_sqlite3_InternalError = caml_named_value("Sqlite3.InternalError");
   caml_sqlite3_Error = caml_named_value("Sqlite3.Error");
@@ -302,7 +318,8 @@ CAMLprim value caml_sqlite3_open(value v_file)
   } else if (db == NULL)
     raise_sqlite3_InternalError(
       "open returned neither a database nor an error");
-  else {
+  /* "open" succeded */
+  {
     db_wrap *dbw;
     value v_res = caml_alloc_final(2, dbw_finalize_gc, 1, 100);
     Sqlite3_val(v_res) = NULL;
@@ -344,7 +361,7 @@ CAMLprim value caml_sqlite3_errmsg(value v_db)
   return caml_copy_string(sqlite3_errmsg(dbw->db));
 }
 
-CAMLprim value caml_sqlite3_last_insert_rowid (value v_db)
+CAMLprim value caml_sqlite3_last_insert_rowid(value v_db)
 {
   db_wrap *dbw = Sqlite3_val(v_db);
   check_db(dbw, "last_insert_rowid");
@@ -373,7 +390,7 @@ static inline int exec_callback(
       v_header = safe_copy_string_array((const char **) header, num_columns);
     End_roots();
 
-    v_ret = callback2_exn(*cbx->cbp, v_row, v_header);
+    v_ret = caml_callback2_exn(*cbx->cbp, v_row, v_header);
 
     if (Is_exception_result(v_ret)) {
       *cbx->exn = Extract_exception(v_ret);
@@ -418,7 +435,7 @@ CAMLprim value caml_sqlite3_exec(value v_db, value v_maybe_cb, value v_sql)
 }
 
 static inline int exec_callback_no_headers(
-  void *cbx_, int num_columns, char **row, char **header)
+  void *cbx_, int num_columns, char **row, char __unused **header)
 {
   struct callback_with_exn *cbx = cbx_;
   value v_row, v_ret;
@@ -481,7 +498,7 @@ static inline int exec_not_null_callback(
       v_header = safe_copy_string_array((const char **) header, num_columns);
     End_roots();
 
-    v_ret = callback2_exn(*cbx->cbp, v_row, v_header);
+    v_ret = caml_callback2_exn(*cbx->cbp, v_row, v_header);
 
     if (Is_exception_result(v_ret)) {
       *cbx->exn = Extract_exception(v_ret);
@@ -523,7 +540,7 @@ CAMLprim value caml_sqlite3_exec_not_null(value v_db, value v_cb, value v_sql)
 }
 
 static inline int exec_not_null_no_headers_callback(
-  void *cbx_, int num_columns, char **row, char **header)
+  void *cbx_, int num_columns, char **row, char __unused **header)
 {
   struct callback_with_exn *cbx = cbx_;
   value v_row, v_ret;
@@ -625,7 +642,7 @@ static inline void prepare_it(
   memcpy(stmtw->sql, sql, sql_len);
   stmtw->sql[sql_len] = '\0';
   stmtw->sql_len = sql_len;
-  rc = sqlite3_prepare_v2(dbw->db, stmtw->sql, sql_len,
+  rc = my_sqlite3_prepare(dbw->db, stmtw->sql, sql_len,
                           &(stmtw->stmt), (const char **) &(stmtw->tail));
   if (rc != SQLITE_OK) raise_sqlite3_current(dbw->db, loc);
   if (!stmtw->stmt) raise_sqlite3_Error("No code compiled from %s", sql);
@@ -674,7 +691,7 @@ CAMLprim value caml_sqlite3_recompile(value v_stmt)
     stmtw->stmt = NULL;
   }
   rc =
-    sqlite3_prepare_v2(stmtw->db_wrap->db, stmtw->sql, stmtw->sql_len,
+    my_sqlite3_prepare(stmtw->db_wrap->db, stmtw->sql, stmtw->sql_len,
                        &(stmtw->stmt),
                        (const char **) &(stmtw->tail));
   if (rc != SQLITE_OK) raise_sqlite3_current(stmtw->db_wrap->db, "recompile");
@@ -708,7 +725,7 @@ CAMLprim value caml_sqlite3_bind_parameter_index(value v_stmt, value v_name)
   char *parm_name = String_val(v_name);
   int index = sqlite3_bind_parameter_index(stmt, parm_name);
   if (!index) caml_raise_not_found();
-  else return Val_int(index);
+  return Val_int(index);
 }
 
 CAMLprim value caml_sqlite3_bind_parameter_count(value v_stmt)
@@ -728,19 +745,19 @@ CAMLprim value caml_sqlite3_bind(value v_stmt, value v_index, value v_data)
       default : return Val_rc(SQLITE_ERROR);
     }
   } else {
-    value field = Field(v_data, 0);
+    value v_field = Field(v_data, 0);
     switch (Tag_val(v_data)) {
-      case 0 : return Val_rc(sqlite3_bind_int64(stmt, i, Int64_val(field)));
-      case 1 : return Val_rc(sqlite3_bind_double(stmt, i, Double_val(field)));
+      case 0 : return Val_rc(sqlite3_bind_int64(stmt, i, Int64_val(v_field)));
+      case 1 : return Val_rc(sqlite3_bind_double(stmt, i, Double_val(v_field)));
       case 2 :
         return Val_rc(sqlite3_bind_text(stmt, i,
-                                        String_val(field),
-                                        caml_string_length(field),
+                                        String_val(v_field),
+                                        caml_string_length(v_field),
                                         SQLITE_TRANSIENT));
       case 3 :
         return Val_rc(sqlite3_bind_blob(stmt, i,
-                                        String_val(field),
-                                        caml_string_length(field),
+                                        String_val(v_field),
+                                        caml_string_length(v_field),
                                         SQLITE_TRANSIENT));
     }
   }
@@ -770,7 +787,7 @@ CAMLprim value caml_sqlite3_column_name(value v_stmt, value v_index)
   CAMLparam1(v_stmt);
   sqlite3_stmt *stmt = safe_get_stmtw("column_name", v_stmt)->stmt;
   int i = Int_val(v_index);
-  range_check(i, sqlite3_data_count(stmt));
+  range_check(i, sqlite3_column_count(stmt));
   CAMLreturn(caml_copy_string(sqlite3_column_name(stmt, i)));
 }
 
@@ -779,7 +796,7 @@ CAMLprim value caml_sqlite3_column_decltype(value v_stmt, value v_index)
   CAMLparam1(v_stmt);
   sqlite3_stmt *stmt = safe_get_stmtw("column_decltype", v_stmt)->stmt;
   int i = Int_val(v_index);
-  range_check(i, sqlite3_data_count(stmt));
+  range_check(i, sqlite3_column_count(stmt));
   CAMLreturn(caml_copy_string(sqlite3_column_decltype(stmt, i)));
 }
 
@@ -800,6 +817,12 @@ CAMLprim value caml_sqlite3_data_count(value v_stmt)
   return Val_int(sqlite3_data_count(stmt));
 }
 
+CAMLprim value caml_sqlite3_column_count(value v_stmt)
+{
+  sqlite3_stmt *stmt = safe_get_stmtw("column_count", v_stmt)->stmt;
+  return Val_int(sqlite3_column_count(stmt));
+}
+
 CAMLprim value caml_sqlite3_column(value v_stmt, value v_index)
 {
   CAMLparam1(v_stmt);
@@ -807,7 +830,7 @@ CAMLprim value caml_sqlite3_column(value v_stmt, value v_index)
   value v_res;
   sqlite3_stmt *stmt = safe_get_stmtw("column", v_stmt)->stmt;
   int len, i = Int_val(v_index);
-  range_check(i, sqlite3_data_count(stmt));
+  range_check(i, sqlite3_column_count(stmt));
   switch (sqlite3_column_type(stmt, i)) {
     case SQLITE_INTEGER :
       v_tmp = caml_copy_int64(sqlite3_column_int64(stmt, i));
